%==========================================================================
%
%   Compute Complex Relection Coefficient of a Layered Medium.
%
%       R_0t = layered_refl_coef(e_r,m_r,D,THETA,f,TE) computes the complex
%   reflection coefficient R_0t of a layered dielectric medium.  The
%   dielectric properties of each layer are contained within the vectors
%   e_r and m_r, which represent the complex relative permittivities (e_r)
%   and real-valued relative permeabilities (m_r) of each layer.  The
%   vector D represents the distance of each planar dielectric boundary
%   from the origin at z = 0 meters.  THETA represents the angle of
%   incidence, and f represents the frequency of the incident plane wave.
%   The boolean TE indicates either transverse electric (TE == 1) or
%   transverse magnetic (TE == 0) polarization.
%
%       The computation assumes an incident plane wave propagating in the
%   -z-direction with an incident angle of THETA (radians) relative to the
%   z-axis (see diagram below).  The z-position of each planar boundary is
%   indicated in the vector D, and has units of meters.  Note that if D has
%   a length of n, then e_r and m_r should both have a length of t = n + 1.
%   The scalar f indicates the frequency of the incident wave in Hertz.
%   The TE argument is a boolean flag that indicates either a transverse
%   electric (TE == 1) wave or a transverse magnetic (TE == 0) wave.  For a
%   TE wave, the incident electric field is polarized along the
%   +y-direction.  For a TM wave, the magnetic field is the one polarized
%   along the +y-direction.
%
%       The reflection coefficient R_0t is assumed to be taken at a
%   reference location of z = 0 meters.  The model automatically assumes
%   that the values in D are all negative, which means the values for D
%   should always be POSITIVE and INCREASING.  An initial value of D(1) = 0
%   is also allowed.
%
%       Note that the imaginary component to the permittivity can be either
%   POSITIVE OR NEGATIVE.  By the convention within this code, a POSITIVE
%   value for the imaginary permittivity is LOSSY, while a NEGATIVE value
%   is AMPLIFYING.  THIS CONVENTION IS NOT UNIVERSAL!  It is therefore
%   important to be sure that the imaginary permittivity fits the
%   convention of this program before computing the reflection coefficient.
%   
%       Also note that amplifying media tend to behave in very quirky ways.
%   If the imaginary component is too large, then the equations will start
%   dividing by zero as the waves amplify out of control.  This makes the
%   results meaningless if the user attempts to push the material too far. 
%
%       See the diagram below for a visualization of how the model is set
%   up.  The computational equations in this program can be found in
%   "Electromagnetic Wave Theory," by Jin Au Kong, pp 385-397.  The
%   notation for naming variables in this program also follow Kong's
%   convention.
%   
%
% R_0t (z = 0) ----                  THETA                   +z
%                 |                 |---/                     | 
%                \|/                |  /                      |
%                                   | /                       |----> +x
%               e_r(1), m_r(1)      |/                          
%                   --------------------------------- z = -D(1)
%               e_r(2), m_r(2)
%                   --------------------------------- z = -D(2)
%                                   .
%                                   .
%                                   .
%               e_r(n), m_r(n)      
%                   --------------------------------- z = -D(n)
%               e_r(t), m_r(t)
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright October 19, 2010
%
%   NOTE!  This code is only validated for TE waves at normal incidence.
%   Do not trust any results for TM waves or nonzero theta.

function R_0t = layered_refl_coef(e_r,m_r,D,theta,f,TE)

%  Physical constants.
e_0 = 8.854187e-12;         %  Free-space permittivity (F/m).
m_0 = 4*pi*1e-7;            %  Free-space permeability (H/m).
eps = e_0*e_r;              %  Layer permittivities.
mu  = m_0*m_r;              %  Layer permeabilities.
n   = length(D);            %  Total number of boundaries.
t   = n + 1;                %  Total number of regions.
w   = 2*pi*f;               %  Angular frequency (rad/s).
k   = w.*sqrt(mu.*eps);     %  Complex wave numbers (1/m).
eta = sqrt(mu./eps);        %  Complex wave impedances (Ohms).

%  Check for length error on inputs.
if n ~= length(e_r) - 1
    error('D not correct length.  Should be length(e_r) - 1.');
end

%  Check to make sure D is properly increasing.
if any(D < 0) || any(diff(D) < 0)
    error(['D must be positive and increasing.']);
end

%  x-components of wavenumbers.  These are all identical due to the
%  phase-matching condition between boundaries.
k_x = k(1)*sin(theta);

%  z-components of k.  See the dispersion relation (eq 3.4.55).
k_z = sqrt( k.^2 - k_x^2 );

%  Kong's convention is a little backwards.  This fixes the wave numbers
%  so that they give proper angles in the reflection coefficients.  Before,
%  I was getting the correct magnitudes but wrong phases.
k_z = -conj(k_z);

%  Solve for p-constants.
if TE

    %  TE case.
    for m = 1:n
        %  Equation 3.4.64 in Kong.
        p(m) = mu(m)*k_z(m+1)/ ( mu(m+1)*k_z(m) );
    end

else

    %  TM Case.
    for m = 1:n
        %  Equation 3.4.70 in Kong.
        p(m) = eps(m)*k_z(m+1)/ ( eps(m+1)*k_z(m) );
    end

end

%  Boundary-region reflection coefficients.
R = ( 1 - p ) ./ ( 1 + p );

%  Total reflection coefficient from region 0 to region t.
R_0t = refl_coef(1,R,k_z,D);

%  Add negative sign for TM waves.
if ~TE
    R_0t = -R_0t;
end

return;


%==========================================================================
%   Recursion function:     R_0t = refl_coef(m,R,k_z,D)
%
%       m = Iteration number.  Terminates when m == length(D).
%       R = Boundary-region reflection coefficients.
%       k_z = z-components of wavenumbers for each region.
%       D = z-position of each planar boundary.
%
%       R_0t = Total reflection coefficient through all layers.
%
%==========================================================================
function R_0t = refl_coef(m,R,k_z,D)

%  Recursion terminator.
if m == length(D)
    R_0t = R(m)*exp(2*i*k_z(m)*D(m));
    return;
end

%  This is the analytical result in the limit as R -> 0.  The regular
%  formula is prone to strange errors when R gets too small, so this fixes
%  them.
% if abs(R(m)) < 1e-6
%     a1 = exp( 2i*k_z(m)*D(m) );
%     a2 = exp( 2i*k_z(m+1)*D(m) );
%     R_0t = (a1/a2)*refl_coef(m+1,R,k_z,D);
%     return;
% end
% 
% % Equation 3.4.68 in Kong.
% a1 = exp( 2i*k_z(m)*D(m) ) / R(m);
% a2 = 1 - 1/(R(m)^2);
% a3 = exp(2i*( k_z(m+1) + k_z(m))*D(m) );
% a4 = (1/R(m))*exp(2i*k_z(m+1)*D(m)) + refl_coef(m+1,R,k_z,D);
% R_0t = a1 + a2*a3/a4;

%  Re-expression of equation 3.4.68 in Kong.  Writing it this way is much
%  more robust against divide-by-zero errors.
r  = refl_coef(m+1,R,k_z,D);            % Recursion term.
a1 = exp( -i*k_z(m+1)*D(m) );
a2 = R(m)*exp( +i*k_z(m+1)*D(m) );
a3 = R(m)*exp( -i*k_z(m+1)*D(m) );
a4 = exp( +i*k_z(m+1)*D(m) );
a5 = exp(2*i*k_z(m)*D(m));
R_0t = (r*a1 + a2)./(r*a3 + a4)*a5;




return

