%==========================================================================
%
%   Compute Complex Transmission Coefficient of a Layered Medium.
%
%       T = layered_tran_coef(e_r,m_r,D,THETA,f,TE) computes the complex
%   transmission coefficient T of a layered dielectric medium.  The
%   properties of each dielectric layer are defined as:
%
%   e_r = Complex relative permittivity of each layer.
%   m_r = Real-valued relative permeability of each layer.  
%   D   = Distance from z = 0 of each planar dielectric boundary (m).
%   THETA = Angle of incidence relative to z-axis (rads).
%   f   = Frequency of the incident plane wave (Hz).
%   TE  = Boolean to indicate either transverse electric (TE == 1) or
%           transverse magnetic (TE == 0) polarization.
%
%       The computation assumes an incident plane wave propagating in the
%   -z-direction (see diagram below).  Note that if D has a length of n,
%   then e_r and m_r should both have lengths of t = n + 1. For a TE wave,
%   the incident electric field is polarized along the +y-direction.  For a
%   TM wave, the magnetic field is the one polarized along the
%   +y-direction. 
%
%       The reflection coefficient R_0t is assumed to be taken at a
%   reference location of z = 0 meters.  The model automatically assumes
%   that the values in D are all negative, which means the values for D
%   should always be POSITIVE and INCREASING.  An initial value of D(1) = 0
%   is also allowed.   
%
%       Note that the imaginary component to the permittivity can be either
%   POSITIVE OR NEGATIVE.  By the convention within this code, a POSITIVE
%   value for the imaginary permittivity is LOSSY, while a NEGATIVE value
%   is AMPLIFYING.  BEWARE!  THIS CONVENTION IS NOT UNIVERSAL!  It is
%   therefore important to be sure that the imaginary permittivity fits the
%   convention of this program before computing the reflection coefficient.
%   
%       Also note that amplifying media tend to behave in very quirky ways.
%   If the imaginary component is too large, then the equations will start
%   dividing by zero as the waves amplify out of control.  This makes the
%   results meaningless if the user attempts to push the material too far.
%
%   [T,p_0t] = layered_tran_coef(...) also returns the dielectric ratio
%   between the incident medium (region 1) and the final transmitting
%   medium (region n).  The total POWER transmittivity is found by using
%   t = real(p_0t)*abs(T)^2.  
%
%       See the diagram below for a visualization of how the model is set
%   up.  The computational equations in this program can be found in
%   "Electromagnetic Wave Theory," by Jin Au Kong, pp 385-397.  The
%   notation for naming variables in this program also follow Kong's
%   convention.
%
%                                    THETA                   +z
%                                   |---/                     | 
%                                   |  /                      |
%                                   | /                       |----> +x
%               e_r(1), m_r(1)      |/                          
%                   --------------------------------- z = -D(1)
%               e_r(2), m_r(2)
%                   --------------------------------- z = -D(2)
%                                   .
%                                   .
%                                   .
%   T ---       e_r(n), m_r(n)      
%       |           --------------------------------- z = -D(n)
%       |       e_r(t), m_r(t)
%      \|/
%
%
%
%==========================================================================
%
%   James Nagel
%   Department of Electrical and Computer Engineering
%   University of Utah, Salt Lake City, Utah
%   nageljr@ieee.org
%   Copyright October 19, 2010
%
%   NOTE!  This code is only validated for TE waves at normal incidence.
%   Do not trust any results for TM waves or nonzero theta.

function [T,t] = layered_tran_coef(e_r,m_r,D,theta,f,TE)

%  Physical constants.
e_0 = 8.854187e-12;         %  Free-space permittivity (F/m).
m_0 = 4*pi*1e-7;            %  Free-space permeability (H/m).
eps = e_0*e_r;              %  Layer permittivities.
mu  = m_0*m_r;              %  Layer permeabilities.
n   = length(D);            %  Total number of boundaries.
t   = n + 1;                %  Total number of regions.
w   = 2*pi*f;               %  Angular frequency (rad/s).
k   = w.*sqrt(mu.*eps);     %  Complex wave numbers (1/m).

%  Check for length error on inputs.
if n ~= length(e_r) - 1
    error('D not correct length.  Should be length(e_r) - 1.');
end

%  Check to make sure D is properly increasing.
if any(D < 0) || any(diff(D) < 0)
    error(['D must be positive and increasing.']);
end

%  x-components of wavenumbers.  These are all identical due to the
%  phase-matching condition between boundaries.
k_x = k(1)*sin(theta);

%  z-components of k.  See the dispersion relation (eq 3.4.55).
k_z = sqrt( k.^2 - k_x^2 );

%  Kong's convention is a little backwards.  This fixes the wave numbers
%  so that they give proper angles in the reflection coefficients.  Before,
%  I was getting the correct magnitudes but wrong phases.
k_z = -conj(k_z);

%  Solve for p-constants.
if TE

    %  TE case.
    for m = 1:n
        %  Equation 3.4.74 in Kong.
        p(m) = mu(m+1)*k_z(m)/ ( mu(m)*k_z(m+1) );
    end

else

    %  TM Case.
    for m = 1:n
        %  Equation 3.4.75 in Kong.
        p(m) = eps(m+1)*k_z(m)/ ( eps(m)*k_z(m+1) );
    end

end

%  Boundary-region reflection coefficients.
R = ( 1 - p ) ./ ( 1 + p );

%  Fill in forward propagating matrices.  See Kong, eq 3.4.78.
V = zeros(2,2,n);
lnV = zeros(2,2,n);
for m = 1:n
    
    if m == n         

        %  Complex phase term.
        a = i*k_z(m+1)*D(m);
        
        %  Let's experiment here by removing the attenuation.
        a = i*imag(a);
        
        %  Fill V-matrix.
        V(1,1,m) =      exp( +a );
        V(1,2,m) = R(m)*exp( +a );
        V(2,1,m) = R(m)*exp( -a );
        V(2,2,m) =      exp( -a );
        
    else
        
        %  Complex phase term.
        a = i*k_z(m+1)*(D(m+1) - D(m));

        %  Fill V-matrix.        
        V(1,1,m) =      exp( -a );
        V(1,2,m) = R(m)*exp( -a );
        V(2,1,m) = R(m)*exp( +a );
        V(2,2,m) =      exp( +a );
        
    end

    %  Kong, eq 3.4.78.
    V(:,:,m) = (1/2)*(1 + p(m))*V(:,:,m);

end

%  Compute the total forward propagating matrix by multiplying all the
%  sub-matrices together.
V_t0 = eye(2,2);
for m = 1:n

    Vx = squeeze(V(:,:,m));
    V_t0 = Vx*V_t0;

end

%  Now compute the total reflection coefficient of the layered medium.  We
%  need this to compute T.
R_0t = layered_refl_coef(e_r,m_r,D,theta,f,TE);

%  Use the total forward-propagating matrix to compute the final forward-
%  and reverse-propagating wave amplitudes (X(1) and X(2), respectively).
X = V_t0*[R_0t*exp(-i*k_z(1)*D(1)); exp(i*k_z(1)*D(1))];
T = X(2);

%  The wave amplitude for T is currently referenced against the point 
%  z = 0.  In reality, the value we're really after is more like S_21 in a
%  two-port network.  To fix this, multiply T by the complex phase-offset
%  between z = 0 and z = -D(end).  This references T against the final
%  dielectric boundary, giving a more meaningful phase and proper
%  attenuation if there is any loss.
%T = T*exp(i*k_z(n+1)*D(n));

%  CORRECTION:  Without the attenuation term, there is no need to offset
%  for attenuation.  So just use real part of k_z to account for phase.
%  This really helps to improve numerical stability with high-loss systems.
T = T*exp(i*real(k_z(n+1))*D(n));

%  Power transmission coefficient.
if TE
    a = real(mu(1)*k_z(end)) / real( mu(end)*k_z(1) );
    t = a*abs(T)^2;
else
    a = real(eps(1)*k_z(end)) / real( eps(end)*k_z(1) );
    t = a*abs(T)^2;
end

%  Something very bizarre seems to happen when materials are too lossy.  In
%  theory, T and t should just be very small.  But somehow some numerical
%  error causes them to result in extremely huge numbers instead.  This
%  seems to fix the issue, but I should really try to figure out exactly
%  what the problem is before slapping a band-aid like this onto it.
if any(abs(V(:)) > 1e12)
    T = 0;
   	t = 0;
end

return;

